class BaseLightmanError(Exception): ...


class BaseConfigError(BaseLightmanError): ...


class InvalidConfigError(BaseConfigError): ...


class ConfigNotFoundError(BaseConfigError): ...


class PromptNotFoundError(BaseConfigError): ...
