DEFAULT_CONFIG_SECTION = "default"

DEFAULT_CONFIG_FILE = "lightman.toml"

DEFAULT_ENV_FILE = ".env"

DEFAULT_LOG_LEVEL = "WARNING"
VERBOSE_LOG_LEVEL = "INFO"

DEFAULT_AGENT = "openai"
DEFAULT_SCORE = 8
DEFAULT_TIME_ZONE = "UTC"
